from urllib.parse import urlencode
from typing import Any, Dict


class QueryBuilder:
    """
    QueryBuilder helps construct query parameters for API requests in a fluent and chainable way.

    Example usage:
        qb = QueryBuilder().eq('name', 'John').gt('age', 18).limit(10).offset(20).build()
        # Result: '?name=John&age_gt=18&limit=10&offset=20'
    """

    def __init__(self):
        """
        Initialize a new QueryBuilder instance.
        """
        self.params: Dict[str, Any] = {}

    def eq(self, field: str, value: Any) -> "QueryBuilder":
        """
        Add an equality filter (field == value).
        """
        self.params[field] = value
        return self

    def contains(self, field: str, value: Any) -> "QueryBuilder":
        """
        Add a filter to check if the field contains the given value.
        """
        self.params[f"{field}_contains"] = value
        return self

    def gt(self, field: str, value: Any) -> "QueryBuilder":
        """
        Add a greater-than filter (field > value).
        """
        self.params[f"{field}_gt"] = value
        return self

    def gte(self, field: str, value: Any) -> "QueryBuilder":
        """
        Add a greater-than-or-equal filter (field >= value).
        """
        self.params[f"{field}_gte"] = value
        return self

    def lt(self, field: str, value: Any) -> "QueryBuilder":
        """
        Add a less-than filter (field < value).
        """
        self.params[f"{field}_lt"] = value
        return self

    def lte(self, field: str, value: Any) -> "QueryBuilder":
        """
        Add a less-than-or-equal filter (field <= value).
        """
        self.params[f"{field}_lte"] = value
        return self

    def limit(self, value: int) -> "QueryBuilder":
        """
        Set a limit on the number of results.
        """
        self.params["limit"] = value
        return self

    def offset(self, value: int) -> "QueryBuilder":
        """
        Set the number of results to skip (useful for pagination).
        """
        self.params["offset"] = value
        return self

    def from_dict(self, filters: Dict[str, Any]) -> "QueryBuilder":
        """
        Add multiple filters from a dictionary.
        """
        for key, value in filters.items():
            self.params[key] = value
        return self

    def build(self, prefix: str = "?") -> str:
        """
        Build and return the query string. Returns an empty string if no parameters are set.
        """
        if not self.params:
            return ""
        return prefix + urlencode(self.params)



from cocobase_client.client import CocoBaseClient
import time

test_api_key = "M-CN05en7o_UGPc9jrQJ0mIbAzJGxZ0qeUJ7KaiA"

cb = CocoBaseClient(api_key=test_api_key)


def timed_update_document():
    start = time.time()
    x = cb.update_document(
        "1",
        "00ee8d9d-ddd6-400a-ba89-a727013bb8de",
        {
            "full_name": "Updated Name",
        },
    )
    end = time.time()
    print(x)
    print(f"Time taken: {end - start:.4f} seconds")


if __name__ == "__main__":
    timed_update_document()



# This file makes the src/cocobase_client directory a package for import in tests.
# You can import from cocobase_client in your test files.

from .client import CocoBaseClient
from .exceptions import CocobaseError, InvalidApiKeyError
from .record import Record
from .query import QueryBuilder



from datetime import datetime
from typing import Any, Optional


class Record:
    """A class representing a record in a database, allowing for dictionary-like access with typed getters."""

    data: dict
    id: str
    collectionId: str
    createdAt: datetime
    collection: dict

    def __init__(self, data: dict):
        self.data = data.get("data", {})
        self.id = data.get("id", "")
        self.collectionId = data.get("collection_id", "")
        created_at = data.get("created_at", 0)
        # Handle string, int, float, or ISO format for created_at
        if isinstance(created_at, datetime):
            self.createdAt = created_at
        elif isinstance(created_at, (int, float)):
            self.createdAt = datetime.fromtimestamp(created_at)
        elif isinstance(created_at, str):
            try:
                # Try ISO format first
                self.createdAt = datetime.fromisoformat(created_at)
            except ValueError:
                try:
                    # Try as float timestamp string
                    self.createdAt = datetime.fromtimestamp(float(created_at))
                except Exception:
                    self.createdAt = datetime.fromtimestamp(0)
        else:
            self.createdAt = datetime.fromtimestamp(0)
        self.collection = data.get("collection", {})

    def __repr__(self):
        return f"Record({self.data})"

    def __getitem__(self, key):
        return self.data[key]

    def __setitem__(self, key, value):
        self.data[key] = value

    def __delitem__(self, key):
        del self.data[key]

    def get(self, key, default=None):
        return self.data.get(key, default)

    def items(self):
        return self.data.items()

    # -----------------------------
    # Typed getter helpers below
    # -----------------------------

    def get_string(self, key: str, raise_error: bool = False) -> Optional[str]:
        value = self.data.get(key)
        try:
            return str(value) if value is not None else None
        except Exception:
            if raise_error:
                raise TypeError(f"Value for '{key}' is not string-convertible: {value}")
            return None

    def get_int(self, key: str, raise_error: bool = False) -> Optional[int]:
        value = self.data.get(key)
        if value is None:
            return None
        try:
            return int(value)
        except Exception:
            if raise_error:
                raise TypeError(f"Value for '{key}' is not int-convertible: {value}")
            return None

    def get_float(self, key: str, raise_error: bool = False) -> Optional[float]:
        value = self.data.get(key)
        if value is None:
            return None
        try:
            return float(value)
        except Exception:
            if raise_error:
                raise TypeError(f"Value for '{key}' is not float-convertible: {value}")
            return None

    def get_bool(self, key: str, raise_error: bool = False) -> Optional[bool]:
        value = self.data.get(key)
        if value is None:
            return None
        try:
            if isinstance(value, bool):
                return value
            if isinstance(value, str):
                val = value.strip().lower()
                if val in ("true", "1", "yes"):
                    return True
                if val in ("false", "0", "no"):
                    return False
            return bool(int(value))  # for numeric truthiness
        except Exception:
            if raise_error:
                raise TypeError(f"Value for '{key}' is not bool-convertible: {value}")
            return None

    def get_datetime(self, key: str, raise_error: bool = False) -> Optional[datetime]:
        value = self.data.get(key)
        try:
            if isinstance(value, datetime):
                return value
            if isinstance(value, (int, float)):
                return datetime.fromtimestamp(value)
            if isinstance(value, str):
                try:
                    return datetime.fromisoformat(value)
                except ValueError:
                    return datetime.fromtimestamp(float(value))
        except Exception:
            pass
        if raise_error:
            raise TypeError(f"Value for '{key}' is not datetime-convertible: {value}")
        return None


class Collection:
    """A class representing a collection in a database, allowing for dictionary-like access with typed getters."""

    name: str
    id: str
    createdAt: datetime

    def __init__(self, data: dict):
        self.name = data.get("name", "")
        self.id = data.get("id", "")
        created_at = data.get("created_at", 0)
        # Handle string, int, float, or ISO format for created_at
        if isinstance(created_at, datetime):
            self.createdAt = created_at
        elif isinstance(created_at, (int, float)):
            self.createdAt = datetime.fromtimestamp(created_at)
        elif isinstance(created_at, str):
            try:
                # Try ISO format first
                self.createdAt = datetime.fromisoformat(created_at)
            except ValueError:
                try:
                    # Try as float timestamp string
                    self.createdAt = datetime.fromtimestamp(float(created_at))
                except Exception:
                    self.createdAt = datetime.fromtimestamp(0)
        else:
            self.createdAt = datetime.fromtimestamp(0)

    def __repr__(self):
        return f"Collection(name={self.name}, id={self.id}, createdAt={self.createdAt})"



import unittest
from cocobase_client.client import CocoBaseClient, CocobaseError
from cocobase_client.query import QueryBuilder
from cocobase_client.record import Record
import uuid

# Replace with your actual API key and test collection/document values
test_api_key = "M-CN05en7o_UGPc9jrQJ0mIbAzJGxZ0qeUJ7KaiA"
test_token = None  # or a valid token if needed


def print_result(test_name, passed):
    print(f"{test_name}-----{'passed' if passed else 'failed'}")


class TestCocoBaseClient(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.client = CocoBaseClient(api_key=test_api_key, token=test_token)
        # Use class variables to share test user info between register and login
        cls.test_email = f"testsuser_{uuid.uuid4().hex[:8]}@example.com"
        cls.test_password = "TestPassword123!"
        # Create a collection for document tests
        try:
            result = cls.client.create_collection("test_collection_for_docs2")
            cls.collection_id = result.id if result else None
        except Exception:
            cls.collection_id = None

    def test_create_collection(self):
        try:
            result = self.client.create_collection("test_collection")
            self.assertIsInstance(result, dict)
            print_result("test_create_collection", True)
        except CocobaseError as e:
            print(f"API Response: {getattr(e, 'args', [''])[0]}")
            print_result("test_create_collection", False)
            raise

    def test_update_collection(self):
        # Use the created collection_id
        try:
            result = self.client.update_collection(
                self.collection_id, collection_name="updated_name"
            )
            self.assertIsInstance(result, dict)
            print_result("test_update_collection", True)
        except CocobaseError as e:
            print(f"API Response: {getattr(e, 'args', [''])[0]}")
            print_result("test_update_collection", False)
            raise

    def test_delete_collection(self):
        # Create and then delete a collection to ensure it exists
        try:
            result = self.client.create_collection("to_delete")
            collection_id = result.id if result else None
            del_result = self.client.delete_collection(collection_id)
            self.assertTrue(del_result)
            print_result("test_delete_collection", True)
        except CocobaseError as e:
            print(f"API Response: {getattr(e, 'args', [''])[0]}")
            print_result("test_delete_collection", False)
            raise

    def test_create_document(self):
        try:
            result = self.client.create_document(self.collection_id, {"field": "value"})
            self.assertIsInstance(result, Record)
            print_result("test_create_document", True)
        except CocobaseError as e:
            print(f"API Response: {getattr(e, 'args', [''])[0]}")
            print_result("test_create_document", False)
            raise

    def test_list_documents(self):
        try:
            result = self.client.list_documents(self.collection_id)
            self.assertIsInstance(result, list)
            print_result("test_list_documents", True)
        except CocobaseError as e:
            print(f"API Response: {getattr(e, 'args', [''])[0]}")
            print_result("test_list_documents", False)
            raise

    def test_authentication(self):
        try:
            self.assertFalse(self.client.is_authenticated())
            print_result("test_authentication", True)
        except Exception:
            print_result("test_authentication", False)
            raise

    def test_register(self):
        try:
            result = self.client.register(self.test_email, self.test_password)
            self.assertTrue(result)
            print_result("test_register", True)
        except CocobaseError as e:
            print(f"API Response: {getattr(e, 'args', [''])[0]}")
            print_result("test_register", False)
            raise

    def test_login(self):
        # Ensure registration happens before login
        self.test_register()
        try:
            result = self.client.login(self.test_email, self.test_password)
            self.assertTrue(result)
            print_result("test_login", True)
        except CocobaseError as e:
            print(f"API Response: {getattr(e, 'args', [''])[0]}")
            print_result("test_login", False)
            raise


if __name__ == "__main__":
    unittest.main()



from requests import Response, get, post, patch, delete
from cocobase_client.config import BASEURL
from cocobase_client.exceptions import CocobaseError
from cocobase_client.query import QueryBuilder
from cocobase_client.record import Collection, Record
from cocobase_client.types import HttpMethod


class CocoBaseClient:
    """
    A client for interacting with the CocoBase API.
    Provides methods for collection, document, and user authentication management.
    """

    api_key = None
    app_client_token = None

    def __init__(self, api_key: str, token: str | None = None):
        """
        Initialize the CocoBaseClient with an API key and optional token.
        """
        self.api_key = api_key
        self.app_client_token = token

    def __request__(
        self,
        url,
        method: HttpMethod = HttpMethod.get,
        data: dict | None = None,
        custom_headers: dict | None = None,
        files=None,
    ) -> Response:
        """
        Internal method to send HTTP requests to the CocoBase API.
        Handles GET, POST, PATCH, and DELETE methods, and allows custom headers.
        """
        headers = {"x-api-key": self.api_key}
        if not url.startswith("/"):
            url = "/" + url
        if custom_headers is not None:
            headers.update(custom_headers)
        if method not in (
            HttpMethod.get,
            HttpMethod.post,
            HttpMethod.delete,
            HttpMethod.patch,
        ):
            raise ValueError(
                "Invalid HTTP method. Use HttpMethod.get, HttpMethod.post, HttpMethod.delete, or HttpMethod.patch."
            )
        url = BASEURL + url
        if method == HttpMethod.get:
            return get(url, headers=headers, files=files)
        elif method == HttpMethod.delete:
            return delete(url, headers=headers, json=data, files=files)
        elif method == HttpMethod.patch:
            return patch(url, headers=headers, json=data, files=files)
        else:
            return post(url, headers=headers, json=data, files=files)

    # ------------------- COLLECTION METHODS -------------------
    def create_collection(
        self, collection_name, webhookurl: str | None = None
    ) -> Collection:
        """
        Create a new collection with an optional webhook URL.
        """
        data = {"name": collection_name}
        if webhookurl is not None:
            data["webhook_url"] = webhookurl
        req = self.__request__("/collections/", method=HttpMethod.post, data=data)
        if req.status_code == 400:
            raise CocobaseError("Invalid Request: " + req.text)
        elif req.status_code == 422:
            raise CocobaseError("A field is missing: " + req.text)
        elif req.status_code == 500:
            raise CocobaseError("Internal Server Error")
        elif req.status_code == 201:
            return Collection(req.json())
        else:
            raise CocobaseError(f"Unexpected status code {req.status_code}: {req.text}")

    def update_collection(
        self,
        collection_id,
        collection_name: str | None = None,
        webhookurl: str | None = None,
    ) -> dict:
        """
        Update an existing collection's name and/or webhook URL.
        """
        data = dict()
        if collection_id is None:
            raise CocobaseError("Collection ID must be provided.")
        if webhookurl is None and collection_name is None:
            raise CocobaseError(
                "At least one of webhook_url or collection_name must be provided."
            )
        if webhookurl is not None:
            data["webhook_url"] = webhookurl
        if collection_name is not None:
            data["name"] = collection_name
        req = self.__request__(
            f"/collections/{collection_id}", method=HttpMethod.patch, data=data
        )
        if req.status_code == 400:
            raise CocobaseError("Invalid Request: " + req.text)
        elif req.status_code == 404:
            raise CocobaseError("Collection not found")
        elif req.status_code == 422:
            raise CocobaseError("A field is missing: " + req.text)
        elif req.status_code == 500:
            raise CocobaseError("Internal Server Error")
        elif req.status_code == 200:
            return req.json()
        else:
            raise CocobaseError(f"Unexpected status code {req.status_code}: {req.text}")

    def delete_collection(self, collection_id) -> bool:
        """
        Delete a collection by its ID.
        """
        if collection_id is None:
            raise CocobaseError("Collection ID must be provided.")
        req = self.__request__(
            f"/collections/{collection_id}", method=HttpMethod.delete
        )
        if req.status_code == 400:
            raise CocobaseError("Invalid Request: " + req.text)
        elif req.status_code == 404:
            raise CocobaseError("Collection not found")
        elif req.status_code == 422:
            raise CocobaseError("A field is missing: " + req.text)
        elif req.status_code == 500:
            raise CocobaseError("Internal Server Error")
        elif req.status_code == 204 or req.status_code == 200:
            return True
        else:
            raise CocobaseError(f"Unexpected status code {req.status_code}: {req.text}")

    # ------------------- DOCUMENT METHODS -------------------
    def create_document(self, collection_id, data: dict) -> Record:
        """
        Create a new document in a collection.
        """
        if collection_id is None:
            raise CocobaseError("Collection ID must be provided.")
        if not isinstance(data, dict):
            raise CocobaseError("Data must be a dictionary.")
        req = self.__request__(
            f"/collections/documents?collection=" + collection_id,
            method=HttpMethod.post,
            data={"data": data},
        )
        if req.status_code == 400:
            raise CocobaseError("Invalid Request: " + req.text)
        elif req.status_code == 404:
            raise CocobaseError("Collection not found")
        elif req.status_code == 422:
            raise CocobaseError("A field is missing: " + req.text)
        elif req.status_code == 500:
            raise CocobaseError("Internal Server Error")
        elif req.status_code == 201:
            return Record(req.json())
        else:
            raise CocobaseError(f"Unexpected status code {req.status_code}: {req.text}")

    def list_documents(
        self, collection_id, query: QueryBuilder | None = None
    ) -> list[Record]:
        """
        List all documents in a collection, optionally filtered by a query.
        """
        if collection_id is None:
            raise CocobaseError("Collection ID must be provided.")
        if query is not None and not isinstance(query, QueryBuilder):
            raise CocobaseError("Query must be an instance of QueryBuilder.")
        url = (
            f"/collections/{collection_id}/documents{query.build()}"
            if query is not None
            else f"/collections/{collection_id}/documents"
        )
        req = self.__request__(url)
        if req.status_code == 400:
            raise CocobaseError("Invalid Request: " + req.text)
        elif req.status_code == 404:
            raise CocobaseError("Collection not found")
        elif req.status_code == 422:
            raise CocobaseError("A field is missing: " + req.text)
        elif req.status_code == 500:
            raise CocobaseError("Internal Server Error")
        elif req.status_code == 200:
            return [Record(doc) for doc in req.json()]
        else:
            raise CocobaseError(f"Unexpected status code {req.status_code}: {req.text}")

    def get_document(self, collection_id, document_id) -> Record:
        """
        Retrieve a single document by its ID from a collection.
        """
        if collection_id is None:
            raise CocobaseError("Collection ID must be provided.")
        if document_id is None:
            raise CocobaseError("Document ID must be provided.")
        req = self.__request__(f"/collections/{collection_id}/documents/{document_id}")
        if req.status_code == 400:
            raise CocobaseError("Invalid Request: " + req.text)
        elif req.status_code == 404:
            raise CocobaseError("Document not found")
        elif req.status_code == 422:
            raise CocobaseError("A field is missing: " + req.text)
        elif req.status_code == 500:
            raise CocobaseError("Internal Server Error")
        elif req.status_code == 200:
            return Record(req.json())
        else:
            raise CocobaseError(f"Unexpected status code {req.status_code}: {req.text}")

    def delete_document(self, collection_id, document_id) -> bool:
        """
        Delete a document from a collection by its ID.
        """
        if collection_id is None:
            raise CocobaseError("Collection ID must be provided.")
        if document_id is None:
            raise CocobaseError("Document ID must be provided.")
        req = self.__request__(
            f"/collections/{collection_id}/documents/{document_id}",
            method=HttpMethod.delete,
        )
        if req.status_code == 400:
            raise CocobaseError("Invalid Request: " + req.text)
        elif req.status_code == 404:
            raise CocobaseError("Document not found")
        elif req.status_code == 422:
            raise CocobaseError("A field is missing: " + req.text)
        elif req.status_code == 500:
            raise CocobaseError("Internal Server Error")
        elif req.status_code == 200 or req.status_code == 204:
            return True
        else:
            raise CocobaseError(f"Unexpected status code {req.status_code}: {req.text}")

    def update_document(self, collection_id, document_id, data: dict) -> Record:
        """
        Update a document in a collection by its ID.
        """
        if collection_id is None:
            raise CocobaseError("Collection ID must be provided.")
        if document_id is None:
            raise CocobaseError("Document ID must be provided.")
        if not isinstance(data, dict):
            raise CocobaseError("Data must be a dictionary.")
        req = self.__request__(
            f"/collections/{collection_id}/documents/{document_id}",
            method=HttpMethod.patch,
            data={"data": data},
        )
        if req.status_code == 400:
            raise CocobaseError("Invalid Request: " + req.text)
        elif req.status_code == 404:
            raise CocobaseError("Document not found")
        elif req.status_code == 422:
            raise CocobaseError("A field is missing: " + req.text)
        elif req.status_code == 500:
            raise CocobaseError("Internal Server Error")
        elif req.status_code == 200:
            return Record(req.json())
        else:
            raise CocobaseError(f"Unexpected status code {req.status_code}: {req.text}")

    # ------------------- AUTHENTICATION METHODS -------------------
    def set_app_client_token(self, app_client_token: str):
        """
        Set the app client token for the client.
        """
        self.app_client_token = app_client_token

    def is_authenticated(self) -> bool:
        """
        Check if the client is authenticated.
        """
        return self.app_client_token is not None

    def register(self, email: str, password: str, data: dict | None = None) -> bool:
        """
        Register a new user with email, password, and optional extra data.
        """
        post_data = {
            "email": email,
            "password": password,
        }
        if data is not None and isinstance(data, dict):
            post_data["data"] = data  # type: ignore
        req = self.__request__(
            "/auth-collections/signup",
            method=HttpMethod.post,
            data=post_data,
        )
        if req.status_code == 200 or req.status_code == 201:
            access_token = req.json().get("access_token")
            if access_token:
                self.set_app_client_token(access_token)
                return True
            else:
                raise CocobaseError(
                    "Registration succeeded but no access token received"
                )
        elif req.status_code == 400:
            raise CocobaseError("Invalid Request: " + req.text)
        elif req.status_code == 409:
            raise CocobaseError("User already exists: " + req.text)
        elif req.status_code == 422:
            raise CocobaseError("A field is missing: " + req.text)
        elif req.status_code == 500:
            raise CocobaseError("Internal Server Error")
        else:
            raise CocobaseError(
                f"Registration failed with status {req.status_code}: {req.text}"
            )

    def login(self, email: str, password: str) -> bool:
        """
        Log in a user with email and password.
        """
        req = self.__request__(
            "/auth-collections/login",
            method=HttpMethod.post,
            data={"email": email, "password": password},
        )
        if req.status_code == 200:
            access_token = req.json().get("access_token")
            if access_token:
                self.set_app_client_token(access_token)
                return True
            else:
                raise CocobaseError("Login succeeded but no access token received")
        elif req.status_code == 400:
            raise CocobaseError("Invalid Request: " + req.text)
        elif req.status_code == 401:
            raise CocobaseError("Invalid credentials: " + req.text)
        elif req.status_code == 422:
            raise CocobaseError("A field is missing: " + req.text)
        elif req.status_code == 500:
            raise CocobaseError("Internal Server Error")
        else:
            raise CocobaseError(
                f"Login failed with status {req.status_code}: {req.text}"
            )

    def logout(self):
        """
        Log out the current user by clearing the app client token.
        """
        self.app_client_token = None

    def get_user_info(self) -> dict:
        """
        Get the current user's information. Requires authentication.
        """
        if not self.is_authenticated():
            raise CocobaseError("Client is not authenticated.")
        req = self.__request__(
            "/auth-collections/user",
            method=HttpMethod.get,
            custom_headers={"Authorization": f"Bearer {self.app_client_token}"},
        )
        if req.status_code == 200:
            return req.json()
        elif req.status_code == 401:
            raise CocobaseError("Unauthorized: Invalid or expired token")
        elif req.status_code == 500:
            raise CocobaseError("Internal Server Error")
        else:
            raise CocobaseError(
                f"Failed to get user info with status {req.status_code}: {req.text}"
            )

    def update_user_info(
        self,
        email: str | None = None,
        password: str | None = None,
        data: dict | None = None,
    ) -> dict:
        """
        Update the current user's information. Requires authentication.
        """
        if not self.is_authenticated():
            raise CocobaseError("Client is not authenticated.")

        form_data = dict()
        if email is not None:
            form_data["email"] = email
        if password is not None:
            form_data["password"] = password
        if data is not None:
            if not isinstance(data, dict):
                raise CocobaseError("Data must be a dictionary.")
            form_data["data"] = data

        if not form_data:
            raise CocobaseError(
                "At least one field (email, password, or data) must be provided."
            )

        req = self.__request__(
            "/auth-collections/user",
            method=HttpMethod.patch,
            data=form_data,
            custom_headers={"Authorization": f"Bearer {self.app_client_token}"},
        )
        if req.status_code == 200:
            return req.json()
        elif req.status_code == 400:
            raise CocobaseError("Invalid Request: " + req.text)
        elif req.status_code == 401:
            raise CocobaseError("Unauthorized: Invalid or expired token")
        elif req.status_code == 422:
            raise CocobaseError("A field is missing: " + req.text)
        elif req.status_code == 500:
            raise CocobaseError("Internal Server Error")
        else:
            raise CocobaseError(
                f"Failed to update user info with status {req.status_code}: {req.text}"
            )



class CocobaseError(Exception):
    """Base class for all Cocobase client errors."""
    pass
class InvalidApiKeyError(CocobaseError):
    """Raised when the provided API key is invalid."""
    def __init__(self, message="Invalid API key provided."):
        super().__init__(message)


from io import BufferedReader
from typing import IO, Optional
from cocobase_client.client import CocoBaseClient
from cocobase_client.exceptions import CocobaseError
from cocobase_client.types import HttpMethod


class CocoFileManager:
    instance: CocoBaseClient = None

    def __init__(self, instance: CocoBaseClient):
        if not instance.api_key:
            raise CocobaseError(
                "cant initialize file manager when api_key has not been set"
            )
        self.instance = instance

    def upload_file(
        self,
        file: IO[bytes],
        filename: Optional[str] = None,
        directory: Optional[str] = None,
    ) -> str:
        """
        uploading files to specified Coco project, optionaly specify directory to choose where on the server to store your files
        """
        # Prepare upload name
        upload_name = f"{directory}/{filename}" if directory and filename else filename

        # Prepare files dict
        files_dict = {"file": (upload_name, file)}

        # Make request
        res = self.instance.__request__(
            "/collections/file",
            method="POST",  # or HttpMethod.post
            files=files_dict,
        )

        # Return uploaded file URL
        return res.json()["url"]


# file_bytes = b"Hello, world!"

# # No content_type provided
# files = {"file": ("myfile.txt", file_bytes)}

# response = requests.post(url, files=files)
